
/****************************************************************************\

    waveapi.cpp 
    Damon Chandler <dmc27@cornell.edu>

    Wrapper functions for reading/writing waveform-audio files.
    Wrapper functions for playing waveform audio files.

 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************


\**************************************************************************/

#include <windows.h>
#pragma hrdstop

#include "waveapi.h"
//><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>


// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//  Check/Initialization Routines
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

bool wavCheck(HRESULT AErrorCode)
{
    return (AErrorCode == AVIERR_OK);
}
//---------------------------------------------------------------------------

void wavInititialize()
{
    AVIFileInit();
}
//---------------------------------------------------------------------------

void wavUninitialize()
{
    AVIFileExit();
}
//---------------------------------------------------------------------------


// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//  Functions for playing a waveform-audio buffer
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

bool wavPlayOpen(HWAVEOUT& AHWavOut, long ACallback, DWORD ANotifyInstance,
    DWORD AOpenFlags, WAVEFORMATEX& AFormatStruct)
{
    return wavCheck(
        waveOutOpen(&AHWavOut, WAVE_MAPPER, &AFormatStruct, ACallback,
                    ANotifyInstance, AOpenFlags)
        );
}
//---------------------------------------------------------------------------

bool wavPlayBegin(HWAVEOUT AHWavOut, WAVEHDR& AWavHdr)
{
    if (wavCheck(waveOutPrepareHeader(AHWavOut, &AWavHdr, sizeof(WAVEHDR))))
    {
		return wavCheck(
            waveOutWrite(AHWavOut, &AWavHdr, sizeof(WAVEHDR))
            );
    }
    return false;
}
//---------------------------------------------------------------------------

void wavPlayEnd(HWAVEOUT AHWavOut, WAVEHDR& AWavHdr)
{
 	waveOutReset(AHWavOut);
	waveOutUnprepareHeader(AHWavOut, &AWavHdr, sizeof(WAVEHDR));
}
//---------------------------------------------------------------------------

void wavPlayClose(HWAVEOUT AHWavOut)
{
	waveOutClose(AHWavOut);
}
//---------------------------------------------------------------------------


// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//  Functions for working with a waveform-audio file
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

bool wavOpenFile(PAVIFILE& ApFile, const char* AFileName,
    unsigned int AMode)
{
    return wavCheck(AVIFileOpen(&ApFile, AFileName, AMode, NULL));
}
//---------------------------------------------------------------------------

bool wavSaveFile(const char* AFileName, PAVISTREAM ApStream,
    WAVEFORMATEX& ApFormatStruct)
{
    AVICOMPRESSOPTIONS CompressionOptions;
    memset(&CompressionOptions, 0, sizeof(AVICOMPRESSOPTIONS));
    CompressionOptions.fccType = streamtypeAUDIO;
    CompressionOptions.lpFormat = &ApFormatStruct;
    CompressionOptions.cbFormat = sizeof(WAVEFORMATEX);
    
    return wavCheck(
        AVISave(AFileName, NULL, NULL, 1, ApStream, &CompressionOptions)
        );
}
//---------------------------------------------------------------------------

void wavCloseFile(PAVIFILE& ApFile)
{
    AVIFileClose(ApFile);
    ApFile = NULL;
}
//---------------------------------------------------------------------------

bool wavGetFileInfo(PAVIFILE ApFile, AVIFILEINFO& AFileInfo)
{
    return wavCheck(AVIFileInfo(ApFile, &AFileInfo, sizeof(AVIFILEINFO)));
}
//---------------------------------------------------------------------------


// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//  Functions for working with a waveform-audio stream
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

bool wavCreateStream(PAVISTREAM& ApStream, PAVIFILE ApFile,
    AVISTREAMINFO& AStreamInfo)
{
    return wavCheck(
        AVIFileCreateStream(ApFile, &ApStream, &AStreamInfo)
        );
}
//---------------------------------------------------------------------------

bool wavOpenStream(PAVISTREAM& ApStream, PAVIFILE ApFile)
{
    return wavCheck(
        AVIFileGetStream(ApFile, &ApStream, streamtypeAUDIO, 0)
        );
}
//---------------------------------------------------------------------------

void wavCloseStream(PAVISTREAM& ApStream)
{
    AVIStreamRelease(ApStream);
    ApStream = NULL;
}
//---------------------------------------------------------------------------

bool wavGetStreamInfo(PAVISTREAM ApStream, AVISTREAMINFO& AStreamInfo)
{
    return wavCheck(
        AVIStreamInfo(ApStream, &AStreamInfo, sizeof(AVISTREAMINFO))
        );
}
//---------------------------------------------------------------------------


// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//  Functions for reading/writing the WAVEFORMATEX structure
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

long wavCalcFormatStructSize(PAVISTREAM ApStream)
{
    long required_bytes = 0;
    AVIStreamFormatSize(ApStream, 0, &required_bytes);
    return required_bytes;
}
//---------------------------------------------------------------------------

bool wavReadFormatStruct(PAVISTREAM ApStream, WAVEFORMATEX& ApFormatStruct)
{
    memset(&ApFormatStruct, 0, sizeof(WAVEFORMATEX));
    long size = wavCalcFormatStructSize(ApStream);
    return wavCheck(
        AVIStreamReadFormat(ApStream, 0, &ApFormatStruct, &size)
        );
}
//---------------------------------------------------------------------------

bool wavWriteStreamFormat(PAVISTREAM ApStream, WAVEFORMATEX& ApFormatStruct)
{
    return wavCheck(
        AVIStreamSetFormat(ApStream, 0, &ApFormatStruct,
                           sizeof(WAVEFORMATEX))
        );
}
//---------------------------------------------------------------------------


// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//  Functions for reading/writing the audio samples
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

long wavCalcBufferSize(PAVISTREAM ApStream)
{
    long required_bytes = 0;
    
    AVISTREAMINFO StreamInfo;
    if (wavGetStreamInfo(ApStream, StreamInfo))
    {
        required_bytes = StreamInfo.dwLength * StreamInfo.dwScale;
    }
    return required_bytes;
}
//---------------------------------------------------------------------------

long wavReadStream(PAVISTREAM ApStream, long AStart, long ANumBytes,
    char* ABuffer)
{
    long bytes_read = 0;

    AVISTREAMINFO StreamInfo;
    if (wavGetStreamInfo(ApStream, StreamInfo))
    {
        long num_samples = static_cast<float>(ANumBytes) /
                           static_cast<float>(StreamInfo.dwScale);
        AVIStreamRead(ApStream, AStart, num_samples, ABuffer, ANumBytes,
                      &bytes_read, NULL);
    }
    return bytes_read;
}
//---------------------------------------------------------------------------

long wavWriteStream(PAVISTREAM ApStream, long AStart, long ANumBytes,
    char* ABuffer)
{
    long bytes_written = 0;

    AVISTREAMINFO StreamInfo;
    if (wavGetStreamInfo(ApStream, StreamInfo))
    {
        long num_samples = static_cast<float>(ANumBytes) /
                           static_cast<float>(StreamInfo.dwScale);
        AVIStreamWrite(ApStream, AStart, num_samples, ABuffer, ANumBytes,
                       AVIIF_KEYFRAME, NULL, &bytes_written);
    }
    return bytes_written;
}
//---------------------------------------------------------------------------


//><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
//  [EOF]
//><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
