
/****************************************************************************\
    -----------------------------------------------------
     mp3Demo.cpp 
     Damon Chandler <dmc27@cornell.edu>
    -----------------------------------------------------

    This sample project demonstrates the use of the MCI in creating an
    audio media player.  As this demo relies on the MCI, it is required
    that the target platform has the correct codecs installed.  Refer
    to the following KB article for more info:
    
    http://support.microsoft.com/support/kb/articles/Q142/7/31.ASP

    This demo uses the supplied MP3Device class (MP3Device.cpp,
    MP3Device.h).  If you are more interested in the specifics of the
    abstracted MCI commands and techniques, refer to the implementation
    of the MCIDevice class (MCIDevice.cpp, MCIDevice.h) and the MCIManip
    library (MCIManip.cpp, MCIManip.h)

\****************************************************************************/

#include <vcl.h>
#pragma hdrstop

#include <math.h>
#include "MP3Device.h"
#include "mp3Demo.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TForm1 *Form1;
MP3Device mp3_;


static const int fixed_height = 105;
static const char fixed_caption[] = "MCI MP3 Player Demo";
//---------------------------------------------------------------------------

// ********** BEGIN MCI-Related Code ****************************************

__fastcall TForm1::TForm1(TComponent* Owner)
    : TForm(Owner), current_pos_(0), track_length_(0)
{
    mp3_.callbackWnd(Handle);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::MMMciNotify(TMessage& AMsg)
{
    UpdateControls();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::UpdateLengthInfo()
{
    current_pos_ = 0;
    track_length_ = mp3_.length() / 1000.0L;
    int mins = floor(track_length_ / 60);
    int secs = track_length_ - (mins * 60);

    LengthText->Caption = " [" + FormatFloat("00", mins) +
                          ":" + FormatFloat("00", secs) +
                          "] total time";
}
//---------------------------------------------------------------------------

void __fastcall TForm1::MP3TimerTimer(TObject *Sender)
{
    long pos = mp3_.position();
    if (pos == mp3_.length())
    {
        current_pos_ = 0;
        mp3_.seek(0);
    }
    else current_pos_ = pos / 1000.0L;
            
    int mins = floor(current_pos_ / 60);
    int secs = current_pos_ - (mins * 60);

    if (track_length_ > 0)
    {
        float scale = static_cast<float>(current_pos_) / track_length_;
        ProgressPanelBar->Width = (ProgressPanel->Width - 3) * scale;
    }

    PositionText->Caption = " [" + FormatFloat("00", mins) +
                            ":" + FormatFloat("00", secs) +
                            "] elapsed time";
    Application->Title = "[" + FileText->Caption.Trim() + "]" +
                         PositionText->Caption;
    if (Height == CaptionPanel->Height)
    {
        CaptionLabel->Caption =
            fixed_caption + AnsiString(" -- ") + PositionText->Caption;
    }
    UpdateControls();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::UpdateControls()
{
    PlayButton->Enabled = !mp3_.playing();
    PauseButton->Enabled = mp3_.playing();
    StopButton->Enabled = mp3_.playing() || mp3_.paused();
    RewButton->Enabled = track_length_ > 0;
    FFButton->Enabled = track_length_ > 0;    
}

//---------------------------------------------------------------------------

void __fastcall TForm1::OpenMP3File(char* filename)
{
    bool was_playing = mp3_.playing();
    if (mp3_.open(filename))
    {
        FileText->Caption =
            "  " + ExtractFileName(AnsiString(filename));
        UpdateLengthInfo();
        MP3Timer->Enabled = true;

        if (was_playing) mp3_.play();
    }
}

//---------------------------------------------------------------------------

void __fastcall TForm1::OpenButtonClick(TObject *Sender)
{
    if (OpenDialog1->Execute())
    {
        OpenMP3File(OpenDialog1->FileName.c_str());
    }
}


//---------------------------------------------------------------------------
/*
void __fastcall TForm1::OpenButtonClick(TObject *Sender)
{
    if (OpenDialog1->Execute())
    {
        bool was_playing = mp3_.playing();
        if (mp3_.open(OpenDialog1->FileName.c_str()))
        {
            FileText->Caption =
                "  " + ExtractFileName(OpenDialog1->FileName);
            UpdateLengthInfo();
            MP3Timer->Enabled = true;

            if (was_playing) mp3_.play();
        }
    }
}
*/
//---------------------------------------------------------------------------

void __fastcall TForm1::PlayButtonClick(TObject *Sender)
{
    mp3_.play();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::StopButtonClick(TObject *Sender)
{
    mp3_.stop();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::PauseButtonClick(TObject *Sender)
{
    mp3_.pause();    
}
//---------------------------------------------------------------------------

void __fastcall TForm1::RewButtonClick(TObject *Sender)
{
    long pos = current_pos_ - 10;
    if (pos >= 0)
    {
        mp3_.pause();
        mp3_.seek(pos * 1000.0L);
        if (!PlayButton->Enabled) mp3_.play();

        current_pos_ = pos;
        float scale = static_cast<float>(current_pos_) / track_length_;
        ProgressPanelBar->Width = ProgressPanel->Width * scale;
    }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::FFButtonClick(TObject *Sender)
{
    long pos = current_pos_ + 10;
    if (pos < track_length_)
    {
        mp3_.pause();
        mp3_.seek(pos * 1000.0L);
        if (!PlayButton->Enabled) mp3_.play();

        current_pos_ = pos;
        float scale = static_cast<float>(current_pos_) / track_length_;
        ProgressPanelBar->Width = ProgressPanel->Width * scale;
    }
}
//---------------------------------------------------------------------------

// ********** END MCI-Related Code ******************************************


void __fastcall TForm1::CloseButtonClick(TObject *Sender)
{
    Close();    
}
//---------------------------------------------------------------------------

void __fastcall TForm1::MinimizeButtonClick(TObject *Sender)
{
    Application->Minimize();    
}
//---------------------------------------------------------------------------

void __fastcall TForm1::CaptionPanelMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
    ReleaseCapture();
    SNDMSG(Handle, WM_NCLBUTTONDOWN, HTCAPTION, 0);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::CaptionPanelDblClick(TObject *Sender)
{
    if (Height == CaptionPanel->Height)
    {
        CaptionLabel->Caption = fixed_caption;
        Height = fixed_height;
        
        RECT RWork;
        if (SystemParametersInfo(SPI_GETWORKAREA, 0, &RWork, 0))
        {
            while (Top + Height > RWork.bottom)
            {
                Top = Top - 3;
                Application->ProcessMessages();
            }
        }
    }
    else
    {
        while (Height > CaptionPanel->Height - 5)
        {
            Height = Height - 10;
            Application->ProcessMessages();
        }
        Height = CaptionPanel->Height;
        
        RECT RWork;
        if (SystemParametersInfo(SPI_GETWORKAREA, 0, &RWork, 0))
        {
            if (abs(Top - RWork.bottom + fixed_height) < Height)
            {
                while (Top < RWork.bottom - Height)
                {
                    Top = Top + 2;
                    Application->ProcessMessages();
                }
            }
        }        
    }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::WMEnterSizeMove(TMessage& AMsg)
{
    DWORD dwPos = GetMessagePos();
    PSnapped_ = Point(LOWORD(dwPos), HIWORD(dwPos));

    SystemParametersInfo(SPI_GETWORKAREA, 0, &RWork_, 0);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::WMMoving(TMessage& AMsg)
{
    DWORD dwPos = GetMessagePos();
    POINT PTrack = {LOWORD(dwPos), HIWORD(dwPos)};
    LPRECT lpRect = reinterpret_cast<LPRECT>(AMsg.LParam);

    static bool IsSnappedX = false;
    static bool IsSnappedY = false;
        
    const int snap_zone = 12;
    if (abs(lpRect->left - RWork_.left) < snap_zone)
    {
        if (!IsSnappedX) PSnapped_ = PTrack;
        IsSnappedX = true;
        lpRect->left = RWork_.left;
        lpRect->right = lpRect->left + Width;
    }
    if (abs(lpRect->top - RWork_.top) < snap_zone)
    {
        if (!IsSnappedY) PSnapped_ = PTrack;
        IsSnappedY = true;
        lpRect->top = RWork_.top;
        lpRect->bottom = lpRect->top + Height;
    }
    if (abs(lpRect->right - RWork_.right) < snap_zone)
    {
        if (!IsSnappedX) PSnapped_ = PTrack;
        IsSnappedX = true;
        lpRect->right = RWork_.right;
        lpRect->left = lpRect->right - Width;
    }
    if (abs(lpRect->bottom - RWork_.bottom) < snap_zone)
    {
        if (!IsSnappedY) PSnapped_ = PTrack;
        IsSnappedY = true;
        lpRect->bottom = RWork_.bottom;
        lpRect->top = lpRect->bottom - Height;
    }

    if (IsSnappedX)
    {
        if (abs(PSnapped_.x - PTrack.x) > snap_zone)
        {
            IsSnappedX = false;
            lpRect->left = lpRect->left + (PTrack.x - PSnapped_.x);
            lpRect->right = lpRect->left + Width;
        }
    }
    if (IsSnappedY)
    {
        if (abs(PSnapped_.y - PTrack.y) > snap_zone)
        {
            IsSnappedY = false;
            lpRect->top = lpRect->top + (PTrack.y - PSnapped_.y);
            lpRect->bottom = lpRect->top + Height;
        }
    }

    TForm::Dispatch(&AMsg);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::SysIconImageClick(TObject *Sender)
{
    PopupMenu1->Popup(Mouse->CursorPos.x, Mouse->CursorPos.y);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::PopupMenu1Popup(TObject *Sender)
{
    Play1->Enabled = PlayButton->Enabled;
    Pause1->Enabled = PauseButton->Enabled;
    Stop1->Enabled = StopButton->Enabled;
}
//---------------------------------------------------------------------------

